﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using Infragistics.Olap;
using System.Collections.Specialized;
using Infragistics.Olap.Data;
using System.Collections;
using System.Threading;

namespace ExpandHierarchyOnDrop
{
    public partial class MainPage : UserControl
    {
        private class NewAreaItemInfo
        {
            public IFilterViewModel FilterViewModel { get; set; }
            public ExecutionContext ExecutionContext { get; set; }
        }

        private IList<NewAreaItemInfo> _newAreaItems = new List<NewAreaItemInfo>();
        private IList<NewAreaItemInfo> _waitToLoadNewAreaItems = new List<NewAreaItemInfo>();

        public MainPage()
        {
            InitializeComponent();
            this.pivotGrid.DataSource.Rows.CollectionChanging += this.Rows_CollectionChanging;
            this.pivotGrid.DataSource.Columns.CollectionChanging += this.Columns_CollectionChanging;
        }



        private void Rows_CollectionChanging(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                this.AddNewAreaItem(e.NewItems, ExecutionContext.Rows);
            }
        }

        private void Columns_CollectionChanging(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                this.AddNewAreaItem(e.NewItems, ExecutionContext.Columns);
            }
        }

        private void DataSource_ResultChanged(object sender, AsyncCompletedEventArgs e)
        {
            if (this._newAreaItems.Count > 0)
            {
                NewAreaItemInfo newAreaItemInfo = this._newAreaItems[0];
                IFilterViewModel filterViewModel = newAreaItemInfo.FilterViewModel;

                this._newAreaItems.RemoveAt(0);

                if (this._newAreaItems.Count == 0)
                {
                    this.pivotGrid.DataSource.ResultChanged -= this.DataSource_ResultChanged;
                }

                foreach (IFilterMember filterMember in filterViewModel.FilterMembers)
                {
                    if (!filterMember.IsLoaded)
                    {
                        this._waitToLoadNewAreaItems.Add(newAreaItemInfo);
                        filterMember.FilterViewModel.LoadFilterMembersCompleted += this.FilterViewModel_LoadFilterMembersCompleted;
                        filterMember.IsExpanded = true;
                    }
                    else if (!filterMember.IsExpanded)
                    {
                        filterMember.IsExpanded = true;
                        ((DataSourceBase)filterViewModel.ViewModel).UpdateVisualMembers(newAreaItemInfo.ExecutionContext);
                        filterViewModel.ViewModel.RefreshGrid();
                    }
                }
            }
        }

        private void FilterViewModel_LoadFilterMembersCompleted(object sender, AsyncCompletedEventArgs e)
        {
            IFilterViewModel filterViewModel = (IFilterViewModel)sender;
            NewAreaItemInfo newAreaItemInfo =
                this._waitToLoadNewAreaItems.FirstOrDefault(ai => ai.FilterViewModel == filterViewModel);

            if (newAreaItemInfo == null)
            {
                return;
            }

            this._waitToLoadNewAreaItems.Remove(newAreaItemInfo);
            filterViewModel.LoadFilterMembersCompleted -= this.FilterViewModel_LoadFilterMembersCompleted;

            ((DataSourceBase)filterViewModel.ViewModel).UpdateVisualMembers(newAreaItemInfo.ExecutionContext);
            filterViewModel.ViewModel.RefreshGrid();
        }

        private void AddNewAreaItem(IList newItems, ExecutionContext executionContext)
        {
            IEnumerable<IAreaItemViewModel> newAreaItems = newItems.Cast<IAreaItemViewModel>();
            foreach (IAreaItemViewModel newAreaItem in newAreaItems)
            {
                IFilterViewModel filterViewModel = newAreaItem as IFilterViewModel;
                if (filterViewModel != null)
                {
                    if (this._newAreaItems.Count == 0)
                    {
                        this.pivotGrid.DataSource.ResultChanged += this.DataSource_ResultChanged;
                    }

                    this._newAreaItems.Add(new NewAreaItemInfo { FilterViewModel = filterViewModel, ExecutionContext = executionContext });
                }
            }
        }
    }
}
